# Generated by roxygen2: do not edit by hand

S3method(IC,targeted)
S3method(coef,cross_validated)
S3method(coef,summary.truncatedscore)
S3method(coef,targeted)
S3method(cv,default)
S3method(cv,learner)
S3method(cv,learner_sl)
S3method(estimate,cate.targeted)
S3method(estimate,learner)
S3method(labels,targeted)
S3method(logLik,targeted)
S3method(model.matrix,design)
S3method(parameter,summary.truncatedscore)
S3method(parameter,targeted)
S3method(plot,calibration)
S3method(predict,calibration)
S3method(predict,density)
S3method(predict,learner)
S3method(predict,naivebayes)
S3method(predict,riskreg.targeted)
S3method(predict,superlearner)
S3method(print,calibration)
S3method(print,cross_validated)
S3method(print,cross_validated.learner_sl)
S3method(print,design)
S3method(print,learner.list)
S3method(print,naivebayes)
S3method(print,summarized_learner)
S3method(print,summary.ate.targeted)
S3method(print,summary.cate.targeted)
S3method(print,summary.moi.targeted)
S3method(print,summary.riskreg.targeted)
S3method(print,summary.targeted)
S3method(print,summary.truncatedscore)
S3method(print,superlearner)
S3method(print,targeted)
S3method(score,cross_validated)
S3method(score,superlearner)
S3method(subset,targeted)
S3method(summary,ate.targeted)
S3method(summary,cate.targeted)
S3method(summary,cross_validated)
S3method(summary,design)
S3method(summary,moi.targeted)
S3method(summary,riskreg.targeted)
S3method(summary,targeted)
S3method(summary,truncatedscore)
S3method(terms,design)
S3method(transform,targeted)
S3method(update,cate.targeted)
S3method(update,design)
S3method(vcov,targeted)
S3method(weights,numeric)
S3method(weights,superlearner)
export(IC)
export(RATE)
export(SL)
export(Surv)
export(aipw)
export(alean)
export(ate)
export(calibrate)
export(calibration)
export(cate)
export(cluster)
export(cumhaz)
export(cv)
export(design)
export(estimate)
export(estimate_truncatedscore)
export(expand.list)
export(isoregw)
export(learner)
export(learner_expand_grid)
export(learner_gam)
export(learner_glm)
export(learner_glmnet_cv)
export(learner_grf)
export(learner_hal)
export(learner_isoreg)
export(learner_mars)
export(learner_naivebayes)
export(learner_sl)
export(learner_stratify)
export(learner_surv_cox)
export(learner_surv_rf)
export(learner_svm)
export(learner_xgboost)
export(metalearner_convexcomb)
export(metalearner_discrete)
export(metalearner_nnls)
export(moi)
export(naivebayes)
export(nondom)
export(parameter)
export(pava)
export(predict_filter_bound)
export(predict_filter_bound_dynamic)
export(riskreg)
export(riskreg_cens)
export(riskreg_fit)
export(riskreg_mle)
export(score)
export(scoring)
export(sim)
export(softmax)
export(solve_ode)
export(specify_ode)
export(strata)
export(stratify)
export(superlearner)
export(test_intersection_sw)
export(test_zmax_onesided)
import(Rcpp)
import(methods)
importFrom(R6,R6Class)
importFrom(grDevices,nclass.Sturges)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(lava,IC)
importFrom(lava,Inverse)
importFrom(lava,estimate)
importFrom(lava,expit)
importFrom(lava,getoutcome)
importFrom(lava,logit)
importFrom(lava,na.pass0)
importFrom(lava,parameter)
importFrom(lava,predict_glm)
importFrom(lava,score)
importFrom(lava,sim)
importFrom(rlang,call_match)
importFrom(rlang,hash)
importFrom(stats,.getXlevels)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov2cor)
importFrom(stats,delete.response)
importFrom(stats,density)
importFrom(stats,deriv)
importFrom(stats,dnorm)
importFrom(stats,drop.terms)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,nlminb)
importFrom(stats,offset)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(stats,weights)
importFrom(survival,Surv)
importFrom(survival,cluster)
importFrom(survival,strata)
importFrom(survival,survfit)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(targeted, .registration=TRUE)
